package app

import (
	"context"
	"net/http"

	"github.com/rs/zerolog"
	"go.opentelemetry.io/otel/trace"
	"google.golang.org/grpc"

	"gitea.libretechconsulting.com/rmcguire/go-app/pkg/config"
	"gitea.libretechconsulting.com/rmcguire/go-app/pkg/srv"
)

type App struct {
	AppContext    context.Context
	HTTP          *AppHTTP
	GRPC          *AppGRPC
	cfg           *config.AppConfig
	l             *zerolog.Logger
	tracer        trace.Tracer
	shutdownFuncs []shutdownFunc
	appDone       chan any
}

type AppGRPC struct {
	Services []*GRPCService
	GRPCOpts []grpc.ServerOption
}

type GRPCService struct {
	Name    string            // Descriptive name of service
	Type    *grpc.ServiceDesc // Type (from protoc generated code)
	Service any               // Implementation of GRPCService.Type (ptr)
}

type AppHTTP struct {
	Funcs        []srv.HTTPFunc
	Middleware   []http.Handler
	HealthChecks []srv.HealthCheckFunc
	httpDone     <-chan any
}

type (
	healthCheckFunc func(context.Context) error
	shutdownFunc    func(context.Context) error
)
